/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007-2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {

dojo.provide("com.ibm.rational.report.web.core.app.view.report.NewReportDesignViewer");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.form.ComboBox");
dojo.require("dijit.form.Button");
dojo.require("dojo.io.iframe");
dojo.require("dojox.data.dom");
dojo.require("dijit.layout.LayoutContainer");
dojo.require("dijit.layout.ContentPane");
dojo.require("dijit.layout.TabContainer");
dojo.require("dojo.parser");
dojo.require("com.ibm.rational.report.web.core.client.internal.ReportClient");
dojo.require("com.ibm.rational.report.web.core.app.internal.MessageFormatter");
dojo.require("com.ibm.rational.report.web.core.app.internal.common.ErrorDialog");
dojo.require("com.ibm.rational.report.web.core.app.internal.common.MessageDialog");
dojo.require("com.ibm.rational.report.web.core.app.internal.Util");
dojo.require("com.ibm.team.repository.web.client.session.Session");
dojo.require("com.ibm.team.repository.web.transport.TeamServerClient");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.rational.report.web.core.app.internal.common.Loading");
dojo.require("com.ibm.rational.report.web.core.app.internal.common.PermissionsWidget");
dojo.require("com.ibm.rational.report.web.core.app.view.report.ReportContent");
dojo.requireLocalization("com.ibm.rational.report.web.core.app.internal.common", "Messages");
dojo.requireLocalization("com.ibm.rational.report.web.core.app.serviceability", "Messages");
dojo.requireLocalization("com.ibm.rational.report.web.core.app.view.report", "Messages");

var CommonMessages	    	= dojo.i18n.getLocalization("com.ibm.rational.report.web.core.app.internal.common", "Messages");
var ServiceabilityMessages	= dojo.i18n.getLocalization("com.ibm.rational.report.web.core.app.serviceability", "Messages");
var ReportMessages			= dojo.i18n.getLocalization("com.ibm.rational.report.web.core.app.view.report", "Messages");
var ReportClient			= com.ibm.rational.report.web.core.client.internal.ReportClient;
var MessageFormatter		= com.ibm.rational.report.web.core.app.internal.MessageFormatter;
var Util					= com.ibm.rational.report.web.core.app.internal.Util;
var session					= com.ibm.team.repository.web.client.session;
var TeamServerClient 		= com.ibm.team.repository.web.transport.TeamServerClient;
var ServiceResponseHandler	= com.ibm.team.repository.web.transport.ServiceResponseHandler;
var PermissionsWidget		= com.ibm.rational.report.web.core.app.internal.common.PermissionsWidget;
var ReportContent			= com.ibm.rational.report.web.core.app.view.report.ReportContent;

var REPORTS_ERROR_DIALOG    = "com.ibm.rational.report.web.core.app.view.admin.ReportsErrorDialog";

var SERVICE_URI				= "com.ibm.rational.report.common.IReportContentRestService/";
var ACTION_PROP				= "__upLoadReportDesignActionProp";

var REPORT_DESIGN			= dojo.moduleUrl("com.ibm.rational.report.web.core", "app/internal/common/icons/report_design.gif");
var REPORT_DESIGN_SHARED	= dojo.moduleUrl("com.ibm.rational.report.web.core", "app/internal/common/icons/report_design_shared.gif");

var tabClickEventHandles = [];

/**
 * Displays the Upload Report Design view.
 */
dojo.declare(
	"com.ibm.rational.report.web.core.app.view.report.NewReportDesignViewer",
	[dijit._Widget, dijit._Templated],	{
	
	templatePath: dojo.moduleUrl("com.ibm.rational.report.web.core", "app/view/report/templates/NewReportDesignViewer.html"),

	startup: function() {
		this.inherited(arguments);
		
		// instantiate widgets used to display tabs
		var nodeList = [];
		nodeList.push(this.layoutContainer);
		nodeList.push(this.mainContentPane);
		nodeList.push(this.tabContainer);
		nodeList.push(this.detailsContentPane);
		if (this._createPermissionsTab) {
			nodeList.push(this.permissionContentPane);
		}
		nodeList.push(this.viewReportContentPane);
		var nodes = dojo.parser.instantiate(nodeList);
		
		var tabContainerId = nodes[2].id;
		this.tabContainerDomNode = nodes[2].domNode;
		dojo.subscribe(tabContainerId+"-selectChild", this, "_newTabSelected");
		
		var tabs = dojo.query(".dijitTabInnerDiv", this.tabContainerDomNode);		
		for (var i = 1; i < tabs.length; i++) {
			var handle = dojo.connect(tabs[i], "onclick", this, "_disableTabs");
			tabs[i].setAttribute("class","dijitTabInnerDiv disabledTabs");
			tabClickEventHandles.push(handle);	
		}
	},
	
	constructor: function(args) {
		this._controller = args.controller;
		this._parentNodeFolderId = args.nodeId;
		this._createPermissionsTab = true;
		if (args.createPermissionsTab != null) {
			this._createPermissionsTab = args.createPermissionsTab;
		}
		this._args = args;
	},
	
	postCreate: function() {
		this._initText();
		
		this._msgFormatter = new MessageFormatter(this.messageArea);
		this._postSave = null;
		this._postCancel = null;		
		this.count = 1;
		this.fileListRowId = 0;
		this.currentInput = this.reportDesignFileInput;
		this._fileListArray = [];
		this.baseDialogWidth = 625;
		this.baseDialogHeight = 325;
		this.deltaBaseWidth = 30;
		this.deltaBaseHeight = 30;
		this.szReportDesigns=null;
		this.szReportDesign=null;
		this.updateReportDesign=false;
		
		dojo.connect(this._reportDesignFilesTable, "onclick", this, "_handleTableClick");
		
		var args = this._args;
		
		if (typeof args.postSave != "undefined") {
			this._postSave = args.postSave;
		}
		
		if (typeof args.postCancel != "undefined") {
			this._postCancel = args.postCancel;
		}
		
		this._newReportDesign = null;
		this._newTreeNode = null;
		this.permissionsWidget = null;
		this.reportContent = null;
		
		var self = this;
		
		this._createTeamAreaCombo();
				
		dojo.connect(this.saveButton, "onclick",
			function(e) {
				e.preventDefault();
				self._handleSave();
			}
		);
		this.saveButton.disabled = true;
		
		dojo.connect(this.closeButton, "onclick",
			function(e) {
				e.preventDefault();
				if (self._postCancel) {
					self._postCancel();
				}
				else {
					self._controller.defaultAction();
				}
			}
		);
		
		dojo.connect(this.nameText, "onkeyup",
			function(e) {
				e.preventDefault();
				var name = dojo.trim(self.nameText.value);
				self.saveButton.disabled = (name.length == 0);
			}
		);		
		
		dojo.connect(this.nameText, "onkeypress",
			function(e) {
				if (e.charCode == 47 || e.charCode == 92 || e.charCode == 42 || e.charCode == 63 ||
				    e.charCode == 124 || e.charCode == 62 || e.charCode == 60 || e.charCode == 58  || e.charCode == 34){
						e.preventDefault();
				}
			}
		);
		
		dojo.connect(this.reportDesignFileInput, "onchange", this, "_addFileInput");
		
		// initialize the report design files table
		var chk = document.createElement("input");
		chk.type = "checkbox";
		
		this._tableColumns = [
			{field: "Select", sortType:"__markup__", label:chk,  width: "12px"},
			{field: "FileName", label: CommonMessages.FILENAME },
			{field: "Master", label: CommonMessages.MASTER, sortType: "__markup__",noSort: true}
		];
		
		var toolbar = new dijit.Toolbar({
			layoutAlign: "top",
			style: "float: left; background : transparent; border: none"
		});
		
		this.deleteButton = new dijit.form.Button({
			title: CommonMessages.DELETE_REPORTS_TOOLTIP,
			iconClass: "DeleteButtonIcon",
			label: CommonMessages.DELETE_REPORTS_TOOLTIP,
			showLabel: false,
			onClick: dojo.hitch(this, this._deleteReportDesignFiles)
		});
		
		this.downloadButton = new dijit.form.Button({
			title: CommonMessages.DOWNLOAD_REPORTS_TOOLTIP,
			iconClass: "DownloadButtonIcon",
			label: CommonMessages.DOWNLOAD_REPORTS_TOOLTIP,
			showLabel: false,
			onClick: dojo.hitch(this, this._downloadReportDesignFiles)
		});
		
		this.downloadButton.disabled = true;
		this.deleteButton.disabled = true;
		this.downloadButton._setStateClass();
		this.deleteButton._setStateClass();
		
		toolbar.addChild(this.deleteButton);
		toolbar.addChild(this.downloadButton);
		
		this._tableColumns.push({
			field: "Toolbar",
			sortType: "__markup__",
			label: toolbar.domNode
		});
				
		this._table = new com.ibm.rational.report.web.core.app.internal.common.FilteringTable(
			{valueField: "rowId", noSelectField: "Select"}, this._reportDesignFilesTable);
				
		for (var i = 0; i < this._tableColumns.length; i++) {
			this._table.columns.push(this._table.createMetaData(this._tableColumns[i]));
		}
		
		dojo.connect(this._table, "onGroupSelect", this, "_onGroupSelect");
		
		if (this._createPermissionsTab == false) {
			this.tabContainer.removeChild(this.permissionContentPane);
		}
		
		dojo.connect(this.reportDesignFileInput, "onkeypress", 
			function(e) {				
				if (e.keyCode == dojo.keys.TAB || e.keyCode == dojo.keys.ENTER) {
					return;
				} else {
					// prevent anything from being entered
					e.preventDefault();	
					return false;
				}
			}
		);	
	},
	
	_initText: function() {
		Util.removeChildren(this.pageTitle);
		var pageTitleIcon = document.createElement("img");
		this.pageTitle.appendChild(pageTitleIcon);
		pageTitleIcon.src = REPORT_DESIGN;
		this.pageTitle.appendChild(document.createTextNode(ReportMessages.NEW_REPORT_DESIGN_VIEWER_PAGE_TITLE));
		
		Util.removeChildren(this.saveButton);
		this.saveButton.appendChild(document.createTextNode(CommonMessages.SAVE_BUTTON_LABEL));
		
		Util.removeChildren(this.closeButton);
		this.closeButton.appendChild(document.createTextNode(CommonMessages.CLOSE_BUTTON_LABEL));
		
		Util.removeChildren(this.nameLabel);
		this.nameLabel.appendChild(document.createTextNode(ReportMessages.NEW_REPORT_DESIGN_VIEWER_NAME_LABEL));
	
		Util.removeChildren(this.descriptionLabel);
		this.descriptionLabel.appendChild(document.createTextNode(ReportMessages.NEW_REPORT_DESIGN_VIEWER_DECRIPTION_LABEL));
		
		Util.removeChildren(this.typeLabel);
		this.typeLabel.appendChild(document.createTextNode(ReportMessages.NEW_REPORT_DESIGN_VIEWER_TYPE_LABEL));
		
		Util.removeChildren(this.teamAreaLabel);
		this.teamAreaLabel.appendChild(document.createTextNode(ReportMessages.NEW_REPORT_DESIGN_VIEWER_TEAM_AREA_LABEL));
	
		Util.removeChildren(this.sharedLabel);
		this.sharedLabel.appendChild(document.createTextNode(ReportMessages.NEW_REPORT_DESIGN_VIEWER_SHARED_LABEL));
	
		Util.removeChildren(this.fileLabel);
		this.fileLabel.appendChild(document.createTextNode(ReportMessages.NEW_REPORT_DESIGN_VIEWER_FILE_LABEL));
		
		Util.removeChildren(this.filePromptLabel);
		this.filePromptLabel.appendChild(document.createTextNode(ReportMessages.NEW_REPORT_DESIGN_VIEWER_FILE_PROMPT_LABEL));
	
		this.detailsContentPane.title = ReportMessages.NEW_REPORT_DESIGN_VIEWER_DETAILS_TAB_TITLE;
		this.permissionContentPane.title = ReportMessages.NEW_REPORT_DESIGN_VIEWER_PERMISSIONS_TAB_TITLE;
		this.viewReportContentPane.title = ReportMessages.NEW_REPORT_DESIGN_VIEWER_REPORT_TAB_TITLE;		
	},
	
	_onGroupSelect : function() {
		var rows = this._table.getGroupSelectedRows();
		this.deleteButton.disabled = (rows <= 0);
		this.deleteButton._setStateClass();	
	},
	
	_addFileInput: function() {
		// handle windows and linux paths.. 
		var fullPath = this.currentInput.value.replace(/\\/g, "\/");
		var parts = fullPath.split("\/");
		var fileName = parts[parts.length - 1];
				
		this.currentInput.style.position = 'absolute';
		this.currentInput.style.left = '-1000px';
								
		var szCurrentInput = this.currentInput;
		
		this.currentInput = document.createElement("INPUT");
		this.currentInput.type = "file";
		this.currentInput.name = 'file_' + this.count++;
		this.currentInput.size = "80";
		var self = this;
		
		this.currentInput.dojoattachpoint = this.currentInput.name;
				
		this._fileInputList.appendChild(this.currentInput);
		
		// check for duplicate report design files
		for (var i = 0; i < this._fileListArray.length; i++) {
			if (fileName == this._fileListArray[i].FileName) {	
				this._displayErrorMessage(ServiceabilityMessages.REPORT_DESIGN_ALREADY_EXISTS);	
				dojo.connect(this.currentInput, "onchange", this, "_addFileInput");	
				return;
			}
		}	
		
		var masterrB;
		
	    if (dojo.isIE) {
			masterrB = document.createElement('<input name=" ">');
		} else if(dojo.isMozilla) {
			masterrB = document.createElement('input');
		}
    	masterrB.type = 'radio';
    	masterrB.id = fileName;
		masterrB.name = "MasterFile";
		
		var selectcB = document.createElement('input');
    	selectcB.type ='checkbox';
    	selectcB.id= fileName;
		
		var reportFileRowData = {
			rowId: this.fileListRowId,
			itemId: this.fileListRowId,
			Select: selectcB,
			FileName: fileName,
			Description: "this._selectedNode.report.itemId",
			Master: masterrB,
			Toolbar:null,
			filePath: fullPath, //local copy
			isNew: true,
			isRemoved: false,
			inputFile: szCurrentInput  //reference to input file
		};
		
		selectcB.reportFileRowData = reportFileRowData;
		masterrB.reportFileRowData = reportFileRowData;			
		
		// increment table row ID
		this.fileListRowId++;  
		
		// push the row data into table
		this._fileListArray.push(reportFileRowData);
		
		this._table.store.addData(reportFileRowData,i);
		
		dojo.connect(this.currentInput, "onchange", this, "_addFileInput");
		
		this.saveButton.disabled = false;
	},
	
	_addReportFileRow: function(reportFileRowData,szResizeWindow) {
		var tbody = this._fileListTableBody;
		var row;
		if (true) {
			row = tbody.insertRow(0);
		} else {
			row= tbody.insertRow(-1);
		}
		var cell = row.insertCell(-1); // file name
		cell.appendChild(document.createTextNode(reportFileRowData.fileName));
		cell = row.insertCell(-1);  // master file radio button
		var index = 0;
		var process;
		
	    if (dojo.isIE) {
			process = document.createElement('<input name=" ">');
		} else if (dojo.isMozilla) {
			process = document.createElement('input');
		}
        process.type ='radio';
        process.name = "MasterFile";
        process.value = index; 
        process.reportFileRowData = reportFileRowData;
        cell.appendChild(process);
					
		index++;
		
		cell = row.insertCell(-1); // actions
		var self = this;
		var removeAnchor = document.createElement("a");			
		removeAnchor.href = "#";
		removeAnchor.className = "actionLink";
		removeAnchor.reportFileRowData = reportFileRowData;
		removeAnchor.appendChild(document.createTextNode(ReportMessages.NEW_REPORT_DESIGN_VIEWER_REMOVE_ANCHOR_LABEL));
		removeAnchor[ACTION_PROP] = "remove";
		cell.appendChild(removeAnchor);
	},
	
	_handleTableClick: function(e) {
		var a = this._getOriginalTarget(e);
		var action = a[[ACTION_PROP]];
		if (action === "remove") {
			e.preventDefault();
			this._performRemove(a.reportFileRowData);
		} else if (action === "undoRemove") {
		} else {
		}
	},
	
	_deleteReportDesignFiles: function() {
		var inputs = this._reportDesignFilesTable.tBodies[0].getElementsByTagName("input");
		for (var i = 0; i < inputs.length; i++) {
			if (inputs[i].checked && inputs[i].type == 'checkbox') {
				this._performRemove(inputs[i].reportFileRowData);
			}
		}
		this._refreshTable();
	},
	
	_performRemove: function(reportFileRowData) {
 	 	for (var i = 0; i < this._fileListArray.length; i++) {
	 		if (this._fileListArray[i].rowId == reportFileRowData.rowId) {
	 			// remove the row from the array
	 			this._fileListArray.splice(i, 1);
	 			
	 			// decrement table row counter and the file counter
	 			this.fileListRowId--;
	 			this.count--;
	 			
	 			// remove the "input" file div element from the form
	 			this._fileInputList.removeChild(reportFileRowData.inputFile);
	 	
	 			break;
	 		}
 		}
	},
	
	_reGenTableRows: function() {
		// remove all current rows
		dojox.data.dom.removeChildren(this._fileListTableBody);
		
		// recreate the table	
		for (var i = 0; i < this._fileListArray.length; i++) {
			this._addReportFileRow(this._fileListArray[i],false);
		}
	},
	
	_refreshTable: function() {
		// remove all current rows	
		this._table.store.clearData();
		
		// recreate the table	
		for (var i = 0; i < this._fileListArray.length; i++) {
			this._table.store.addData(this._fileListArray[i],i);
		}
		this.downloadButton.disabled = true;
		this.deleteButton.disabled = true;
		this.downloadButton._setStateClass();
		this.deleteButton._setStateClass();	
	},
	
	_downloadReportDesignFiles: function() {	
	},
	
	_createTeamAreaCombo: function() {
		this._projectArea = com.ibm.team.process.web.client.projectarea.getCurrentProjectArea();
		this._parseTeamAreas(this._projectArea.children);
		
		if (this.teamAreaCombo.options.length > 0) {
			this.teamAreaCombo.options[0].selected = true;
		}
		else {
			this.teamAreaDisplay.style.display = "none";
		}
	},
	
	_parseTeamAreas: function(teamAreas) {	
		if (teamAreas == null || teamAreas.length == 0) {
			return;
		}
		for (var i = 0; i < teamAreas.length; i++) {
			var teamArea = teamAreas[i];
			var option = new Option(teamArea.name, teamArea.itemId, false, false);	
			this.teamAreaCombo.options[this.teamAreaCombo.options.length] = option;		
			this._parseTeamAreas(teamArea.children);
		}
	},
		
	_handleSave: function() {
		var name = this.nameText.value;
  		for (var i = 0; i < name.length; i++) {
			if (name.charCodeAt(i) == 47 || name.charCodeAt(i) == 92 || name.charCodeAt(i) == 42 || name.charCodeAt(i) == 63 ||
			    name.charCodeAt(i) == 124 || name.charCodeAt(i) == 62 || name.charCodeAt(i) == 60 || name.charCodeAt(i) == 58  || name.charCodeAt(i) == 34){
				this._displayErrorMessage(CommonMessages.ERROR_FILE_NAME+ ": * \\\ / | \  : < \" > ? ");
				return;
			}
		}
		
		var reportDesignFiles = 0;
		var reportDesignType = this.typeCombo.value;
		if (reportDesignType == "BIRT") {
			for (var i = 0; i < this._fileListArray.length; i++) {
				var fileName = this._fileListArray[i].FileName;
				var fileParts = fileName.split(".");
				var fileExt = fileParts[fileParts.length - 1];
				if (fileExt.toLowerCase() == "rptdesign") {
					reportDesignFiles++;
				}
			}
		}
		else if (reportDesignType == "CRYSTAL_REPORTS") {
			for (var i = 0; i < this._fileListArray.length; i++) {
				var fileName = this._fileListArray[i].FileName;		
				var fileParts = fileName.split(".");
				var fileExt = fileParts[fileParts.length - 1];
				if (fileExt.toLowerCase() == "rpt") {
					reportDesignFiles++;
				}
			}
		}
		
		if (reportDesignFiles == 0) {
			this._displayErrorMessage(ServiceabilityMessages.PROMPT_FOR_UPLOAD_REPORT_DESIGN_FILES);
			return;
		}
		
		if (this._newTreeNode == null) {
			this._createReportDesign();
		}
		else {
			if (this.permissionsWidget != null) {
				this.permissionsWidget._updateAclEntries();
			}
			this.saveButton.disabled = true;
		}
	},
	
	_createReportDesign: function() {
		var self = this;
		
		var name = this.nameText.value;			
		if (name.length == 0) {
			this._displayErrorMessage(ServiceabilityMessages.PROMPT_FOR_REPORT_DESIGN_NAME);
			return;
		}		
		
		Util.removeChildren(this.messageArea);
		
		var inputs = this._reportDesignFilesTable.tBodies[0].getElementsByTagName("input");
		
		if (this.fileListRowId < 1) {	
			this._displayErrorMessage(ServiceabilityMessages.PROMPT_FOR_UPLOAD_REPORT_DESIGN_FILES);
			return;
		}
			
		if (this.fileListRowId == 1) {
			var szRadioButton=inputs[0].reportFileRowData.Master;
			this.masterReportInput.value = szRadioButton.id;
		}
		
		var masterReportDesignSelection = false;
		
		if (this.fileListRowId > 1) {
			for (var i = 0; i < inputs.length; i++) {
				if (inputs[i].type == 'radio') {
					var radioButton = inputs[i].reportFileRowData.Master;
					if (radioButton.checked) {
						this.masterReportInput.value = radioButton.id;
						masterReportDesignSelection = true;
					}	
				}
			}
			
			if (!masterReportDesignSelection) {	
				this._displayErrorMessage(ServiceabilityMessages.PROMPT_FOR_MASTER_DESIGN_FILE);
				return;
			}
		}
		
		var srh = new ServiceResponseHandler(this, "_handleReportCreation", "_remoteError");
		ReportClient.getReportDesignExistsInServer(srh, {name: name});
	},
	
	_handleReportCreation: function(report) {
		var description = this.descriptionText.value;
		var type = this.typeCombo.value;
		var projectAreaId = this._projectArea.itemId;
		var teamAreaId = (this.teamAreaDisplay.style.display == "none") ?
			"" : this.teamAreaCombo.options[this.teamAreaCombo.selectedIndex].value;
		var ownerId = session.getAuthenticatedContributor().itemId;
		var shared = this.sharedCheckbox.checked;
		var name = this.nameText.value;
	
		if (this.updateReportDesign) {
			var srh = new ServiceResponseHandler(this, "_done",	"_remoteError");		
			ReportClient.postUpdateReportDesign(srh,
				{id: report.itemId, name: name, description: description,
				 projectAreaId: projectAreaId,teamAreaId: teamAreaId, shared: shared});
			this._uploadReportDesignContent(report);
		}		
		else {
			var srh = new ServiceResponseHandler(this, "_uploadReportDesignContent", "_remoteError");
			ReportClient.postNewReportDesign(srh,
				{name: name, description: description, type: type,
				 projectAreaId: projectAreaId, teamAreaId: teamAreaId, ownerId: ownerId, shared: shared});
		}
	},
	
	_uploadReportDesignContent: function(reportDesign) {
		this._newReportDesign = reportDesign;
		this.createReportDesignForm.action = TeamServerClient._getServicePrefix() + SERVICE_URI;				
		this.reportDesignIdInput.value = reportDesign.itemId;
	
		var self = this;
		var count = 0;
		var headers = {};
		headers["content-type"] = "application/x-www-form-urlencoded; charset=utf-8";
		var bindArgs = {
			form: document.getElementById("createReportDesignForm"),
			handleAs: "text",
			headers: headers,		
			handle: function(data,ioArgs){
				if(data == "OK"){
					self._createTreeNode();
					Util.removeChildren(self.messageArea);
				}
				else{
					self._msgFormatter.displayErrorMessage(data,null);
				}
			}
		};
		var request = dojo.io.iframe.send(bindArgs);	
		Util.removeChildren(this.messageArea);
		this.saveButton.disabled = true;
	},
	
	_createTreeNode:function() {
		if (this.updateReportDesign) {
			// Check if we're updating. If we are, we need to check if a report design already exists in this folder.
			// To do this check, retrieve all the nodes in this folder.
			var reportDesignId = this._newReportDesign.itemId;
			var parentNodeFolderId = this._parentNodeFolderId;
			var projectAreaId = this._projectArea.itemId;
		    var ownerId = session.getAuthenticatedContributor().itemId;
			var params = {projectAreaId: projectAreaId, ownerId: ownerId, parentId: parentNodeFolderId};
			var responseHandler = new ServiceResponseHandler(this, "_retrievedNodes", "_retrievedNodesFailure");
			ReportClient.getReportTreeNodes(responseHandler, params);
		}
		else {
			// we're creating a new report design that doesn't exist
			this._retrievedNodes(null);
		}
	},
	
	_retrievedNodesFailure: function(error){
		this._msgFormatter.displayErrorMessage(ServiceabilityMessages.FAILED_TO_UPLOAD_REPORT_DESIGN, error);
	},
	
	_retrievedNodes: function(children){
		var name = this.nameText.value;
		var flag = true; // value of variable decides whether we need to create a node in the folder 
		if(children != null){
			for (var i in children) {
				var child = children[i];
				if(child.name == name){
					flag = false;
					break;
				}	
			}
		}
		if (flag) {
			// report design doesn't exist in this folder, so create a node
			var reportDesignId = this._newReportDesign.itemId;
			var parentNodeFolderId = this._parentNodeFolderId;
			var projectAreaId = this._projectArea.itemId;
			var ownerId = session.getAuthenticatedContributor().itemId;
			var params = { reportItemId: reportDesignId, parentFolderId: parentNodeFolderId, projectAreaId: projectAreaId, ownerId: ownerId };
			var responseHandler = new ServiceResponseHandler(this, "_done",	"_remoteError");
			ReportClient.postNewReportTreeNode(responseHandler, params);
		}
	},
	
	_getReportDesigns: function(reportDesigns) {
		this.szReportDesigns=reportDesigns;
	},
	
	_getAllReportDesignsError: function(error) {
		this._msgFormatter.displayErrorMessage(ServiceabilityMessages.FAILED_TO_RETRIEVE_REPORT_DESIGNS, error);
	},
	
	_getFileName: function(fileName) {
		var result = fileName;
		var fileNameSeparatorIndex = fileName.lastIndexOf("\\");
		if (fileNameSeparatorIndex > -1) {
			return fileName.substring(fileNameSeparatorIndex+1, fileName.length);
		}			
		return result;
	},
	
	_getOriginalTarget: function(e) {
		// normalize w3c (originalTarget) and srcElement (IE)
		return e.target || e.srcElement;
	},
	
	_done: function(treeNode) {
		Util.removeChildren(this.messageArea);
		this._controller.syncReportNodeChanged();
		this._controller.reportTreeParser.expandFolder(this._parentNodeFolderId);
		this._newTreeNode = treeNode;
		
		//moving to the edit view of the report design
		this._controller.editReportNode(this._newTreeNode, this._newReportDesign, this._postSave, this._postCancel);
	},
	
	_postUpdateReportDesignError: function(error) {
		var msg = dojo.string.substitute(ServiceabilityMessages.FAILED_TO_UPDATE_REPORT_DESIGN, [dojo.trim(this._reportDesign.name)]);
		this._msgFormatter.displayErrorMessage(msg, error);
	},
	
	_remoteError: function(error) {
		this._msgFormatter.displayErrorMessage(ServiceabilityMessages.FAILED_TO_UPLOAD_REPORT_DESIGN, error);
	},
	
	_updatePageTitle: function(){
		Util.removeChildren(this.pageTitle);				
		var pageTitleIcon = document.createElement("img");
		this.pageTitle.appendChild(pageTitleIcon);
		pageTitleIcon.src = this._newReportDesign.shared ? REPORT_DESIGN_SHARED : REPORT_DESIGN;		
		this.pageTitle.appendChild(document.createTextNode(this._newReportDesign.name));
	},
	
	_displayErrorMessage: function(errorMsg) {
		var errorDialog = new com.ibm.rational.report.web.core.app.internal.common.ErrorDialog();
		errorDialog.displayError(errorMsg);	
	},
	
	_newTabSelected: function(tab) {
		if (tab.dojoAttachPoint == "permissionContentPane") {
			if (this.permissionsWidget == null && this._createPermissionsTab == true) {
				this.permissionsWidget = new PermissionsWidget({
					repositoryItemId: this._newReportDesign.itemId,
					messageArea: this.messageArea,
					resourceType: this._newTreeNode.nodeType
				});
				this.permissionContentPane.appendChild(this.permissionsWidget.domNode);
			}
		}
		else if (tab.dojoAttachPoint == "viewReportContentPane") {
			if (this.reportContent == null) {
				this.reportContent = new ReportContent({
					reportItemId: this._newReportDesign.itemId,
					design: null,
					reportParamNames: null,
					reportParamValues: null,
					collapseParameters: false,
					controller: this._controller,
					type: this._newTreeNode.nodeType,
					parentWidget: this
				});
				this.viewReportContentPane.appendChild(this.reportContent.domNode);
			}
		}	
	},
	
	_disableTabs: function (e){
		e.preventDefault();
		e.stopPropagation();
	}
	
});

})();